Package: broadSeq
Title: broadSeq : for streamlined exploration of RNA-seq data
Version: 1.7.0
Authors@R: 
    person("Rishi", "Das Roy", , "rishi.dasroy@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3276-7279"))
Description: This package helps user to do easily RNA-seq data analysis with multiple 
    methods (usually which needs many different input formats). Here the user will 
    provid the expression data as a SummarizedExperiment object and will get results 
    from different methods. It will help user to quickly evaluate different methods. 
License: MIT + file LICENSE
URL: https://github.com/dasroy/broadSeq
BugReports: https://github.com/dasroy/broadSeq/issues
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
biocViews: GeneExpression, DifferentialExpression, RNASeq,
        Transcriptomics, Sequencing, Coverage, GeneSetEnrichment, GO
Suggests: knitr, limma (>= 3.54.0), rmarkdown, stats (>= 4.2.2), samr
Depends: dplyr, ggpubr, SummarizedExperiment
Imports: BiocStyle, DELocal, EBSeq (>= 1.38.0), DESeq2 (>= 1.38.2),
        NOISeq, forcats (>= 1.0.0), genefilter, ggplot2, ggplotify,
        plyr, clusterProfiler (>= 4.8.2), pheatmap, sechm (>= 1.6.0),
        stringr, purrr (>= 0.3.5), edgeR (>= 3.40.1)
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/broadSeq
git_branch: devel
git_last_commit: 4cc0a21
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:37:45 UTC; biocbuild
Author: Rishi Das Roy [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3276-7279>)
Maintainer: Rishi Das Roy <rishi.dasroy@gmail.com>
